/*
  This example creates a BLE central that scans for a peripheral with a Test Service
  If that contains floatValue characteristics the value can be seen in the Serial Monitor or Plotter.

  The circuit:
  - Arduino Nano 33 BLE or Arduino Nano 33 IoT board.

  This example code is in the public domain.
*/

#include <Adafruit_GFX.h>
#include <Adafruit_SH110X.h>
#define SCREEN_WIDTH 128  // OLED display width, in pixels
#define SCREEN_HEIGHT 128 // OLED display height, in pixels
#define OLED_RESET -1     // can set an oled reset pin if desired
Adafruit_SH1107 display = Adafruit_SH1107(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET, 1000000, 100000);

//----------------------------------------------------------------------------------------------------------------------
// BLE UUIDs
//----------------------------------------------------------------------------------------------------------------------

float val = -1;
float valStep = 0.025;
bool up = true; 

void setup()
{
  //while ( !Serial );
  display.begin(0x3D, false); // Address 0x3D default
  display.display();
  delay(2000);
  display.clearDisplay();
}


void loop()
{
  if(val >= 1){
    up = false;
    val = val - valStep;
  }else if(val <= -1){
    up = true;
  }
  if(up == true){
    val = val + valStep;
  }else{
    val = val - valStep;
  }
  display.clearDisplay();
  float temperatureDiff = val;
  drawGraph(max(min(temperatureDiff,1),-1)*20, "C", 20, 5); //unit, real value to show, scale (f.e. display 10% as 1, where +/-1 is the max. value the graph can show), top position
  float humidityDiff = val*-1;
  drawGraph(max(min(humidityDiff,1),-1)*0.5, "%", 0.5, 80);
  display.display();
}

void drawGraph(float val, String unit, float scale, int marginTop) {
  display.setTextSize(2);
  display.setTextColor(SH110X_WHITE);
  int16_t x1, y1;
  uint16_t w, h;
  int x = 1;
  int y = marginTop;
  display.getTextBounds(String(val)+unit, x, y, &x1, &y1, &w, &h); //calc width of new string
  display.setCursor(x - w/2 + display.width()/2, y);
  display.print(String(val)+unit);
  display.drawLine(display.width()/2, marginTop+25, display.width()/2, marginTop+40+5, SH110X_WHITE);
  display.drawRect(display.width()/2, marginTop+31, ((display.width()-2)*val/scale*0.5), 10, SH110X_WHITE);
}
