// Board Library : Seeed nRF52 mbed-enable Borads 2.8.1
// Board Select  : Seeed nRF52 mbed-enable Borads / Seeed XIAO BLE - nRF52840
// 2022/12/23      Sometimes Peripheral disconnections may not be detected
//                 Sometimes RSSI values are not being retrieved properly
//                 Sometimes connection takes a long time --> need to reset central and peripherals
//**********************************************************************************************
// [IMPORTANT] Battery voltage not readable bug fix
// #define analogPinToPinName(P)  
// /Arduino15/packages/Seeeduino/hardware/mbed/2.7.2/cores/arduino/pinDefinitions.h:22
// g_APinDescription ---> g_AAnalogPinDescription
//**********************************************************************************************
// peripheral : nRF52_XIAO_Attitude_peripheral_1.ino
//              nRF52_XIAO_Attitude_peripheral_2.ino

#include <Wire.h>
#include <Adafruit_GFX.h>         // Adafruit GFX Library 1.10.14 by Adafruit
#include <ArduinoBLE.h>           // ArduinoBLE 1.3.1 by Arduino
#include <Adafruit_SH110X.h>
#define SCREEN_WIDTH 128  // OLED display width, in pixels
#define SCREEN_HEIGHT 128 // OLED display height, in pixels
#define OLED_RESET -1     // can set an oled reset pin if desired
Adafruit_SH1107 display = Adafruit_SH1107(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET, 400000, 100000);

#define VBAT_LOWER  3.5           // battery voltage lower limit
#define VBAT_UPPER  4.2           // battery voltage upper limit
#define dataNum 16                // receive data number : roll, humidity, temperature, Vbatt (4 data 8 byte)

const char* versionNumber = "0.90"; // version number
int err;                            // error code of scan_connect function
bool LED_state;                     // LED ON/OFF state 

float temperatureDiff, humidityDiff;
float temperatureScale = 2;
float humidityScale = 5;

struct prphData {                   // data struct for peripheral
  float roll, humidity, temperature;             // attitude
  float Vbatt;                        // battery voltage
  int ss;                             // peripheral signal strength
  bool readingFlag = false;           // data buffer in use flag
  union unionData {                   // Union for bit convertion 16 <--> 8
    float   dataBuff32[dataNum/4];
    uint8_t   dataBuff8[dataNum];
  };
  union unionData ud;
};
prphData sd_1;                      // data struct for peripheral 1
prphData sd_2;                      // data struct for peripheral 2

// UUID and Device
BLEDevice peripheral_1;
#define myUUID_1(val) ("0dd7eb5a-" val "-4f45-bcd7-94c674c3b25f")
//BLEService        AttService_1(myUUID_1("0000"));
BLECharacteristic dataCharacteristic_1(myUUID_1("0010"), BLEWritten, dataNum);

BLEDevice peripheral_2;
#define myUUID_2(val) ("f78d0557-" val "-4cbc-9ab3-89fc6514877f")
//BLEService        AttService_2(myUUID_2("0000"));
BLECharacteristic dataCharacteristic_2(myUUID_2("0010"), BLEWritten, dataNum);

//***************************************************************************************
void setup() {
  //initialize serial port
  Serial.begin(115200);
  //while (!Serial) { delay(1000); }
  delay(1000);

  //set I/O pins
  pinMode(LED_RED, OUTPUT);       // LOW:LED ON, HIGH:LED OFF
  pinMode(LED_GREEN, OUTPUT);
  digitalWrite(LED_RED, HIGH);
  digitalWrite(LED_GREEN, HIGH);

  display.begin(0x3D, false); // Address 0x3D default
  display.display();
  delay(2000);
  display.clearDisplay();
  display.setTextSize(2);
  display.setTextColor(SH110X_WHITE);

  // initialize BLE
  BLE.begin();
}

//******************************************************************************************
// IMU data is read every 80mS
//
void loop() {
  // scan and connect the peripheral
  // return value of this function is an error code  
    display.setTextSize(2);
    displayText("connecting...", 1000, 10);
  do {   
    err = scan_connect();
    Serial.print("ERROR : ");
    Serial.println(err);
  } while(err != 0);
    Serial.println("go!");
    displayText("connected!", 2000, 10);
    // main task while connected to the peripheral 60~78mS
    while (peripheral_1.connected() & peripheral_2.connected()) {
      LED_state = !LED_state;
      //digitalWrite(LED_GREEN, (LED_state ? LOW : HIGH));   // connect indicator blinking
      
      long timestamp = millis();    // for loop timer
      
      // restoration of received data
      // prohibit event handler access while data buffer is in use          
      sd_1.readingFlag = true;   
        sd_1.roll = sd_1.ud.dataBuff32[0];
        sd_1.humidity = sd_1.ud.dataBuff32[1];
        sd_1.temperature = sd_1.ud.dataBuff32[2];
        sd_1.Vbatt = sd_1.ud.dataBuff32[3] / 1000.0;
      sd_1.readingFlag = false;
      
      sd_2.readingFlag = true;   
        sd_2.roll = sd_2.ud.dataBuff32[0];
        sd_2.humidity = sd_2.ud.dataBuff32[1];
        sd_2.temperature = sd_2.ud.dataBuff32[2];
        sd_2.Vbatt = sd_2.ud.dataBuff32[3] / 1000.0;
      sd_2.readingFlag = false;
      
      // for serial plotter
      //Serial.println("ss1: " + (String)sd_1.ss + " ss2: " + (String)sd_1.ss);
      Serial.print(sd_1.humidity); Serial.print(", ");
      Serial.print(sd_1.temperature); Serial.print(", ");
      Serial.print(sd_2.humidity); Serial.print(", ");
      Serial.println(sd_2.temperature);
      
      temperatureDiff = sd_2.temperature - sd_1.temperature;
      humidityDiff = sd_2.humidity - sd_1.humidity;

      displayData();
      
      while(millis() < timestamp + 10);

    } //While connected


  //if disconnected, scan_connect again
  Serial.println("disconnected");
  Serial.print(peripheral_1.connected());
  Serial.print(peripheral_2.connected());
  Serial.println("  disconnect, and scan_connect again");
  BLE.disconnect();

  //digitalWrite(LED_GREEN, HIGH);            // connect indicator OFF
      
} //loop

//**************************************************************************************************
// scan and connect peripherals
// return value of this function is an error code
//
int scan_connect(void) {

  // scanning peripheral_1   +++++++++++++++++++++++++++++++++++++++++++++++++
  BLE.scanForUuid(myUUID_1("0000"));
  Serial.println("1_1.SCANNING sensor_wing_left ................");
 
  peripheral_1 = BLE.available();
  if (!peripheral_1) {
    Serial.println("1_2x.Peripheral sensor_wing_left unavailable");
    return 12;
  }
  Serial.println("1_2.Peripheral sensor_wing_left is available");

  if (peripheral_1.localName() != "sensor_wing_left") {
    Serial.println("1_3x.Peripheral sensor_wing_left local name miss match");
    return 13;
  }
  Serial.println("1_3.Got the right peripheral sensor_wing_left");
  
  // scanning peripheral_2   +++++++++++++++++++++++++++++++++++++++++++++++++++++ 
  BLE.scanForUuid(myUUID_2("0000"));
  Serial.println("2_1.SCANNING sensor_wing_right ................");
  
  peripheral_2 = BLE.available();
  if (!peripheral_2) {
    Serial.println("2_2x.Peripheral sensor_wing_right unavailable");
    return 22;
  }
  Serial.println("2_2.Peripheral sensor_wing_right is available");

  if (peripheral_2.localName() != "sensor_wing_right") {
    Serial.println("2_3x.Peripheral sensor_wing_right local name miss match");
    return 23;
  }
  Serial.println("2_3.Got the right peripheral sensor_wing_right");

  // stop scanning, connect the peripheral  ++++++++++++++++++++++++++++++++++++++++++++++
  BLE.stopScan();
  Serial.println("4.Stop scanning");
  
  // connect the peripheral_1   +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  Serial.println("1_5.CONNECTING sensor_wing_left ................");
  if (!peripheral_1.connect()) {
    Serial.println("1_5x.Can't connect sensor_wing_left");
    return 15;
  } 
  Serial.println("1_5.Connected sensor_wing_left");

  if (!peripheral_1.discoverAttributes()) {
    Serial.println("1_6x.Didn't discover attributes sensor_wing_left");
    peripheral_1.disconnect();
    return 16;
  }
  Serial.println("1_6.Discovered attributes sensor_wing_left");

  dataCharacteristic_1 = peripheral_1.characteristic(myUUID_1("0010"));       //dataCaracteristic UUID  
  dataCharacteristic_1.setEventHandler(BLEWritten, characteristicWritten_1);  //BLEWritten handler
  Serial.println("1_7.Char and eventhandler setting sensor_wing_left");  

  if (!dataCharacteristic_1.canSubscribe()) {
    Serial.println("1_8x.Can't subscribe sensor_wing_left");
    peripheral_1.disconnect();
    return 18;
  }
  Serial.println("1_8.Can subscribe sensor_wing_left");

  if (!dataCharacteristic_1.subscribe()) {
    Serial.println("1_9x.Can't Subscribe sensor_wing_left");
    peripheral_1.disconnect();
    return 19;
  }
  Serial.println("1_9.Subscribed sensor_wing_left");

  // connect the peripheral_2   +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  Serial.println("2_5.CONNECTING sensor_wing_right ................");
  if (!peripheral_2.connect()) {
    Serial.println("2_5x.Can't connect sensor_wing_right");
    return 25;
  } 
  Serial.println("2_5.Connected sensor_wing_right");
  
  if (!peripheral_2.discoverAttributes()) {
    Serial.println("2_6x.Didn't discover attributes sensor_wing_right");
    peripheral_2.disconnect();
    return 26;
  }
  Serial.println("2_6.Discovered attributes sensor_wing_right");

  dataCharacteristic_2 = peripheral_2.characteristic(myUUID_2("0010"));       //dataCaracteristic UUID
  dataCharacteristic_2.setEventHandler(BLEWritten, characteristicWritten_2);  //BLEWritten handler
  Serial.println("2_7.Char and eventhandler setting sensor_wing_right");  

  if (!dataCharacteristic_2.canSubscribe()) {
    Serial.println("2_8x.Can't subscribe sensor_wing_right");
    peripheral_2.disconnect();
    return 28;
  }
  Serial.println("2_8.Can subscribe sensor_wing_right");

  if (!dataCharacteristic_2.subscribe()) {
    Serial.println("1_9x.Can't Subscribe sensor_wing_right");
    peripheral_2.disconnect();
    return 29;
  }
  Serial.println("2_9.Subscribed sensor_wing_right");

  Serial.println("10.Success scanning and connecting sensor_wing_left and sensor_wing_right");
  return 0;
}

//****************************************************************************************************
// Characteristic written event handler for peripheral 1
//
void characteristicWritten_1(BLEDevice peripheral_1, BLECharacteristic thisChar) {
  //digitalWrite(LED_RED, LOW);         // event indicator ON
    
  // wait while data buffer is accessed in main loop  
  while(sd_1.readingFlag == true) {
  }    
  dataCharacteristic_1.readValue(sd_1.ud.dataBuff8, dataNum);  // read data packet  
  int ss = peripheral_1.rssi();                                // exclude meaningless 0 and 127
  if ((ss != 0) & (ss != 127)) {
    sd_1.ss = ss;
  }  
}

//****************************************************************************************************
// Characteristic written event handler for peripheral 2
//
void characteristicWritten_2(BLEDevice peripheral_2, BLECharacteristic thisChar) {
  //digitalWrite(LED_RED, LOW);         // event indicator ON
    
  // wait while data buffer is accessed in main loop  
  while(sd_2.readingFlag == true) {
  }    
  dataCharacteristic_2.readValue(sd_2.ud.dataBuff8, dataNum);  // read data packet  
  int ss = peripheral_2.rssi();
  if ((ss != 0) & (ss != 127)) {                               // exclude meaningless 0 and 127
    sd_2.ss = ss;
  }
}

void drawGraph(float val, char unit, float scale, int marginTop) {
  int16_t x1, y1;
  uint16_t w, h;
  int x = 1;
  int y = marginTop;
  display.getTextBounds(String(val)+unit, x, y, &x1, &y1, &w, &h); //calc width of new string
  display.setCursor(x - w/2 + display.width()/2, y);
  display.print(val);
  display.print(unit);
  display.drawFastVLine(display.width()/2, marginTop+23, 10, SH110X_WHITE);
  display.drawRect(display.width()/2, marginTop+26, ((display.width()-2)*val/scale*0.5), 5, SH110X_WHITE);
}

void displayData(){
  //display.clearDisplay();
  display.fillScreen(0);
  display.setTextSize(2);
  drawGraph(temperatureDiff, 'C', temperatureScale, 2); //unit, real value to show, scale (f.e. display 10% as 1, where +/-1 is the max. value the graph can show), top position
  drawGraph(humidityDiff, '%', humidityScale, 45);
  display.setTextSize(1);
  display.setCursor(1, 110);
  display.print(sd_1.ss);
  display.print("dB");
  display.setCursor(30 + display.width()/2, 110);
  display.print(sd_2.ss);
  display.print("dB");
  display.setCursor(1, 120);
  display.print(sd_1.Vbatt);
  display.print('V');
  display.setCursor(24 + display.width()/2, 120);
  display.print(sd_2.Vbatt);
  display.print('V');
  display.display();
}

void displayText(String text, int duration, int marginTop){
  int16_t x1, y1;
  uint16_t w, h;
  int x = 1;
  int y = marginTop;
  display.clearDisplay();
  display.getTextBounds(text, x, y, &x1, &y1, &w, &h); //calc width of new string
  display.setCursor(x - w/2 + display.width()/2, y);
  display.print(text);
  display.display();
  delay(duration);
}
