//----------------------------------------------------------------------------------------------
// Board Library : Seeed nRF52 mbed-enabled Borads 2.7.2
// Board Select  : Seeed nRF52 mbed-enabled Borads / Seeed XIAO BLE Sense - nRF52840
// 2022/10/07
//----------------------------------------------------------------------------------------------
// Modification required for battery voltage reading 
// /Arduino15/packages/Seeeduino/hardware/mbed/2.7.2/cores/arduino/pinDefinitions.h:22
// #define analogPinToPinName(P)
//    g_APinDescription ---> g_AAnalogPinDescription
// https://forum.seeedstudio.com/t/xiao-ble-sense-mbed-2-7-2-battery-charge-and-voltage-monitor-analogread-p0-31-does-not-work/266438
//**********************************************************************************************

#include <Wire.h>
#include <ArduinoBLE.h>     // ArduinoBLE 1.3.2 by Arduino   
#include "SHTSensor.h"
SHTSensor sht;

#define AVENUM      32      // average number of ADC
#define HICHG       P0_13   // charge current setting pin　Open:50mA Lo:100mA
#define CHG         P0_17   // charge indicator pin　Open:discharge　Lo:charge(LED ON)
#define VBAT_ENABLE P0_14   // battery vontage read enable　Open:disable　Lo:enable
#define VBAT_READ   P0_31   // battery voltage monitor pin
#define VBAT_LOWER  3.5     // battery voltage lower limit
#define VBAT_UPPER  4.2     // battery voltage upper limit
#define dataNum 16          //send data number : rolll, pitch, yaw, Vbatt (4 data 8 byte)

//LSM6DS3 myIMU(I2C_MODE, 0x6A); // IMU
//Madgwick  filter;              // Madgwick filter

const char* versionNumber = "0.90"; // version number
float Vbatt;                        // battery voltage
uint8_t readData;                   // for reading IMU register
uint32_t timestamp;                 // for delay function
union unionData {                   //Union for bit convertion 16 <--> 8
  float   dataBuff32[dataNum/4];
  uint8_t   dataBuff8[dataNum];
};
union unionData ud;

//generated Linux command uuidgen "0dd7eb5a-xxxx-4f45-bcd7-94c674c3b25f"
#define myUUID(val) ("0dd7eb5a-" val "-4f45-bcd7-94c674c3b25f")
BLEService        AttService(myUUID("0000"));
BLECharacteristic dataCharacteristic(myUUID("0010"), BLERead | BLENotify, dataNum);

void setup() {
  //initialize serial port
  Serial.begin(115200);
  Wire.begin();
  if (sht.init()) {
      Serial.print("init(): success\n");
  } else {
      Serial.print("init(): failed\n");
  }
  sht.setAccuracy(SHTSensor::SHT_ACCURACY_HIGH); // only supported by SHT3x

  //set I/O pins
  pinMode(LED_RED, OUTPUT);       // LOW:LED ON, HIGH:LED OFF
  pinMode(LED_GREEN, OUTPUT);
  pinMode(HICHG, OUTPUT);
  pinMode(CHG, INPUT);
  pinMode(VBAT_ENABLE, OUTPUT);
  pinMode(VBAT_READ, INPUT);
  digitalWrite(LED_RED, HIGH);
  digitalWrite(LED_GREEN, HIGH);
  //digitalWrite(HICHG, LOW);         // charge current 100mA
  digitalWrite(VBAT_ENABLE, LOW);   // battery voltage read enable

  // initialize ADC 2.4V/4096
  analogReference(AR_INTERNAL2V4);  // Vref=2.4V
  analogAcquisitionTime(AT_10_US);
  analogReadResolution(12);         // 4096

  // initialize BLE
  if (!BLE.begin()) {
    Serial.println("starting BLE module failed!");
    while (1);
  }

  // set the local name
  BLE.setLocalName("sensor_wing_left");
  // set the device name
  BLE.setDeviceName("XIAO nRF52840 Sence");
  // set the UUID for the service
  BLE.setAdvertisedService(AttService);
  // add the characteristic to the service  
  AttService.addCharacteristic(dataCharacteristic);
  // add service  
  BLE.addService(AttService);

  // start advertising
  BLE.setAdvertisingInterval(160);    //0.625mS * 160 = 100mS
  BLE.setConnectionInterval(6, 3200); //1.25mS * 6 = 7.5mS, 1.25mS * 3200 = 4S
  BLE.advertise();

}

void loop() {
  // connect the sentral
  Serial.println("Connecting to Central ........");
  BLEDevice central = BLE.central();

  if (central) {    // connected to Central? 
  
    // while connected Central
    Serial.println("Connected Central");
    while(central.connect()) {    

      // battery voltage averasing  32 averaging 1 ~ 6mS
      int Vadc = 0;
      for (int i = 0; i < AVENUM; i++) {        
        Vadc = Vadc + analogRead(VBAT_READ);    // analogRead() 32uS       
      }
      
      Vadc = Vadc / AVENUM;
      Vbatt = 2.961 * 2.4 * Vadc / 4096 * 1.0196;     // Vref=2.4, 1/attination=(510e3 + 1e6)/510e3=2.961
                                                      // correction=1.0196(option)

      
      
      //digitalWrite(LED_RED, HIGH);  // data read and send task indicator OFF
      if(sht.readSample()){
        ud.dataBuff32[1] = sht.getHumidity(), 2; 
        ud.dataBuff32[2] = sht.getTemperature(), 2;        
        ud.dataBuff32[3] = Vbatt * 1000;
      }

      // write to Characteristic as byte data        
      dataCharacteristic.writeValue(ud.dataBuff8, dataNum);   // 40~100uS
      
    } //while connected
      
    // if disconnected from Central
    //digitalWrite(LED_GREEN, HIGH);                // connect indicator OFF
    Serial.print("Disconnected from central: ");
    Serial.println(central.address());
  } //if connect
    
} //loop()
